/*
	$Id: JobAgentSettingsData.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class holds information about plugin settings for 
	a particular job agent.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.JobAgentSettings
	@see <a href="../../../../../../html/developer/api/data_api.plugins.html">Developer documentation: Plug-ins, jobs and job agents</a>
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (fr, 21 okt 2011) $
	@hibernate.class table="`JobAgentSettings`" lazy="true"
*/
public class JobAgentSettingsData
	extends BasicData
{

	public JobAgentSettingsData()
	{}
	
	private JobAgentData jobAgent;
	/**
		The job agent these settings belong to.
		@hibernate.many-to-one column="`jobagent_id`" not-null="true" outer-join="false" update="false"
	*/
	public JobAgentData getJobAgent()
	{
		return jobAgent;
	}
	public void setJobAgent(JobAgentData jobAgent)
	{
		this.jobAgent = jobAgent;
	}
	
	private PluginDefinitionData pluginDefinition;
	/**
		The plugin these settings are used for.
		@hibernate.many-to-one column="`plugindefinition_id`" not-null="true" outer-join="false" update="false"
	*/
	public PluginDefinitionData getPluginDefinition()
	{
		return pluginDefinition;
	}
	public void setPluginDefinition(PluginDefinitionData pluginDefinition)
	{
		this.pluginDefinition = pluginDefinition;
	}
	
	private Boolean trusted;
	/**
		If the plugin should be trusted or not. If the value is null 
		the setting specified by the plugin definition should be used.
		@see PluginDefinitionData#isTrusted()
		@hibernate.property column="`trusted`" type="boolean" not-null="false"
	*/
	public Boolean isTrusted()
	{
		return trusted;
	}
	public void setTrusted(Boolean trusted)
	{
		this.trusted = trusted;
	}
	
	private Long maxMemory;
	/**
		The max amount of memory the plugin can use. If the value is null 
		the setting specified by the plugin definition should be used.
		@see PluginDefinitionData#getMaxMemory()
		@hibernate.property column="`max_memory`" type="long" not-null="false"
	*/
	public Long getMaxMemory()
	{
		return maxMemory;
	}
	public void setMaxMemory(Long maxMemory)
	{
		this.maxMemory = maxMemory;
	}
	
	private int priorityBoost;
	/**
		The priority boost for this plugin. Ie. allows certain plugins to
		go ahead in the job queue.
		@hibernate.property column="`priority_boost`" type="int" not-null="true"
	*/
	public int getPriorityBoost()
	{
		return priorityBoost;
	}
	public void setPriorityBoost(int priorityBoost)
	{
		this.priorityBoost = priorityBoost;
	}
	

}
