/*
	$Id: RoleData.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;
import java.util.HashSet;

/**
	This class holds information about a role.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Role
	@see <a href="../../../../../../html/developer/api/data_api.authentication.html">Developer documentation: User authentication</a>
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
	@hibernate.class table="`Roles`" lazy="false"
*/
public class RoleData
	extends BasicData
	implements NameableData, RemovableData, SystemData, RegisteredData
{

	public RoleData()
	{}
	
	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	@Override
	public String getName()
	{
		return name;
	}
	@Override
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	@Override
	public String getDescription()
	{
		return description;
	}
	@Override
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	@Override
	public boolean isRemoved()
	{
		return removed;
	}
	@Override
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------
	/*
		From the SystemData interface
		-------------------------------------------
	*/
	private String systemId;
	@Override
	public String getSystemId()
	{
		return systemId;
	}
	public void setSystemId(String systemId)
	{
		this.systemId = systemId;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	@Override
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	private Set<UserData> users;
	/**
		Get the set that manages which users are members of this role.
		@hibernate.set table="`UserRoles`" lazy="true"
		@hibernate.collection-key column="`role_id`"
		@hibernate.collection-many-to-many column="`user_id`" class="net.sf.basedb.core.data.UserData"
	*/
	public Set<UserData> getUsers()
	{
		if (users == null) users = new HashSet<UserData>();
		return users;
	}
	void setUsers(Set<UserData> users)
	{
		this.users = users;
	}

	private Set<RoleKeyData> keys;
	/**
		This is the inverse end.
		@see RoleKeyData#getRoles()
		@hibernate.set table="`RoleKeys`" lazy="true"
		@hibernate.collection-key column="`role_id`"
		@hibernate.collection-many-to-many column="`key_id`" class="net.sf.basedb.core.data.RoleKeyData"
	*/
	Set<RoleKeyData> getKeys()
	{
		return keys;
	}
	void setKeys(Set<RoleKeyData> keys)
	{
		this.keys = keys;
	}
	
	private boolean isDefault;
	/**
		If this role should be assigned to new users by default or not.
		@since 2.4
		@hibernate.property column="`is_default`" type="boolean" not-null="true"
	*/
	public boolean isDefault()
	{
		return isDefault;
	}
	public void setDefault(boolean isDefault)
	{
		this.isDefault = isDefault;
	}
	
}
