/**
	$Id$

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.migrate;

import java.io.IOException;
import java.io.Writer;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;


/**
	Generic writer implementation that writes date values from a result
	set. This writer use {@link ResultSet#getDate(int)} to read data from 
	the result set and the {@link DateFormat#format(java.util.Date)} 
	to format the value.
	
	@author Nicklas
	@since 3.1
	@base.modified $Date$
*/
public class DateWriter
	extends AbstractResultWriter
{
	private final DateFormat dateFormat;
	
	public DateWriter(ResultSet rs, int index, Writer writer, char[] nullEscape, DateFormat dateFormat)
	{
		super(rs, index, writer, nullEscape);
		this.dateFormat = dateFormat;
	}

	@Override
	public void write() 
		throws SQLException, IOException
	{
		Date value = rs.getDate(index);
		if (value == null)
		{
			writer.write(NULL_ESCAPE);
		}
		else
		{
			writer.write(dateFormat.format(value));
		}
	}
	
}
