/**
	$Id: EventHandler.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.events;

import java.util.EventListener;

import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionPoint;

/**
	An event handler receives events about things happening to 
	extensions and extension points. For example, when they
	are registered or updated. An event handler is usually paired with
	an {@link EventFilter} that sorts out which types of event that should be
	sent to the event handler.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (to, 11 sep 2008) $
*/
public interface EventHandler
	extends EventListener
{
	
	/**
		Handle an event. Since the types of events are likely to expand in the
		future, we recommend that the event handler is written so that it
		only responds to specific event types or that it is paired with an
		{@link EventFilter} that filters out undesired events.
		
		@param event The event that happend
		@param extensionPoint The extension point that the extension
			extends, or that the event happened to
		@param extension The extension the event happend to, or null if the
			event happended to the extension point
	*/
	public void handleEvent(EventType event, 
			ExtensionPoint<?> extensionPoint, Extension<?> extension);

}
