/**
	$Id$

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager.processor;

import net.sf.basedb.util.extensions.manager.ExtensionsFileProcessor;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;
import net.sf.basedb.util.extensions.manager.ProcessResults;

/**
	File processor implementation that reset the error flag for
	all extension files. This is usually needed to be able to
	re-install a extension that had an error that has now been fixed.

	@author Nicklas
	@since 3.2.4
	@base.modified $Date$
*/
public class ResetErrorFlagProcessor
	implements ExtensionsFileProcessor
{

	private final ProcessResults results;
	private int numFiles;
	
	public ResetErrorFlagProcessor(ProcessResults results)
	{
		this.results = results;
	}
	
	/*
		From the ExtensionsFileProcessor interface
		------------------------------------------
	*/
	@Override
	public void begin(ExtensionsManager manager, int numFiles)
	{
		this.numFiles = 0;
	}
	
	@Override
	public void processFile(ExtensionsManager manager, WriteableExtensionsFile wFile)
	{
		numFiles++;
		wFile.setError(false);
		results.forgetOldResults(wFile.getExtensionsFile());
	}

	@Override
	public void done(ExtensionsManager manager)
	{}

	@Override
	public void done(ExtensionsManager manager, Throwable t)
	{}
	// ---------------------------------------------

	/**
		Get the number of files that was processed.
	*/
	public int getNumFiles()
	{
		return numFiles;
	}

	
}
