/**
	$Id: LessConverter.java 5385 2010-08-13 10:48:17Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jep.convert;

import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.jep.Jep;

import org.nfunk.jep.Node;

/**
	Convert the '&lt;' operator
	to a Query API {@link Restriction}.

	@author nicklas
	@version 2.4
	@base.modified $Date: 2010-08-13 12:48:17 +0200 (fr, 13 aug 2010) $
*/
public class LessConverter
	extends AbstractJepConversionFunction<Node>
{
	
	public LessConverter()
	{}
	
	/*
		From the JepConversionFunction interface
		-------------------------------------------
	*/
	@Override
	public boolean isOperator()
	{
		return true;
	}
	/**
		@return The string '&lt;'
	*/
	@Override
	public String getName()
	{
		return "<";
	}
	@Override
	public String getDescription()
	{
		return "Check if one expression is less than another";
	}
	/**
		Convert to restriction by calling {@link Restrictions#lt(net.sf.basedb.core.query.Expression, net.sf.basedb.core.query.Expression)}.
	*/
	@Override
	public Restriction toRestriction(Node node)
	{
		return Restrictions.lt(
				Jep.nodeToExpression(node.jjtGetChild(0)),
				Jep.nodeToExpression(node.jjtGetChild(1))
			);
	}
	// -------------------------------------------

}
