/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.List;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.SyncFilter;

/**
	A source item transformer factory is responsible for creating
	{@link SourceItemTransformer}:s that can transform between
	the supported {@link #getSupportedSourceItems(SyncFilter.SourceItemTransform)}
	to the specified {@link #getTargetItem()}.
	
	@author Nicklas
	@since 3.5
*/
public interface SourceItemTransformerFactory 
{

	/**
		Get the target item type of this transformer factory. All transformers
		created by this factory are expected to have a matching
		{@link SourceItemTransformer#getTargetItemType()}.
	*/
	public Item getTargetItem();
	
	/**
		Get the supported source item types when transforming in the specified
		direction. The {@link #create(Item, SyncFilter.SourceItemTransform)} method should
		be able to create a {@link SourceItemTransformer} for all of the item types
		returned by this method.
	*/
	public List<Item> getSupportedSourceItems(SyncFilter.SourceItemTransform transform);
	
	/**
		Create a source item transformed that can transform between the given 
		source item and the {@link #getTargetItem()}. Calling this method
		with an unsupported combination should result in a null return value.
	*/
	public SourceItemTransformer create(Item sourceItemType, SyncFilter.SourceItemTransform transform);
	
}
