/**
	$Id: NodeAttribute.java 6455 2014-05-09 08:31:05Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview;

import java.util.Set;

import net.sf.basedb.core.Extract;

/**
	Defines per-node attributes that can be used to store
	extra information together with a node that doesn't fit
	the regular properties.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2014-05-09 10:31:05 +0200 (fr, 09 maj 2014) $
	@see Node#setAttribute(NodeAttribute, Object)
	@see Node#getAttribute(NodeAttribute)
*/
public class NodeAttribute<T>
{
	
	/**
		Store a reference to an extract on a node. This is used when passing
		a common part of the tree so that we later know which branch to follow.
		Eg. when starting on a raw bioassay we should only follow the path with 
		the same extract from the physical bioassay as is on the raw bioassay.
	*/
	public static NodeAttribute<Extract> EXTRACT = new NodeAttribute<Extract>();
	
	/**
		Store the ID values of extracts that should be followed on
		upstream paths as long as the current extract is not among
		one of the given parent extracts.
		@since 3.3
	*/
	public static NodeAttribute<Set<Integer>> EXTRACT_PATH = new NodeAttribute<Set<Integer>>();

	
}
