/**
$Id: ZipFilePacker.java 6127 2012-09-14 07:28:35Z nicklas $

Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

This file is part of BASE - BioArray Software Environment.
Available at http://base.thep.lu.se/

BASE is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

BASE is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.util.zip;

import net.sf.basedb.util.FileUtil;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
	This class packs files into a ZIP archive.

	@author Martin, Nicklas
	@version 2.4
	@base.modified $Date$
*/
public class ZipFilePacker 
	implements FilePacker
{
	private ZipOutputStream zip;
	
	/**
		Create a new packer that compresses files with the zip format.
	*/
	public ZipFilePacker()
	{}

	/*
		From the FilePacker interface
		-------------------------------------------
	*/
	/**
		@return Always "Zip-archive (.zip)"
	*/
	@Override
	public String getDescription()
	{
		return "Zip-archive (.zip)";
	}
	/**
	  @return Always "zip"
	*/
	@Override
	public String getFileExtension()
	{
		return "zip";
	}
	/**
	  	@return Always "application/zip"
	*/
	@Override
	public String getMimeType()
	{
		return "application/zip";
	}
	/**
	 * Wrap the output stream in a {@link ZipOutputStream}.
	 */
	@Override
	public void setOutputStream(OutputStream out)
	{
		this.zip = new ZipOutputStream(out);
		zip.setMethod(ZipOutputStream.DEFLATED);
	}
	/**
		Create a new {@link ZipEntry} and write the compressed
		data to it.
	*/
	@Override
	public void pack(String entryName, InputStream in, long size, long lastModified)
		throws IOException
	{
		boolean isDirectory = in == null;
		if (isDirectory && !entryName.endsWith("/")) entryName += "/";
		ZipEntry entry = new ZipEntry(entryName);
		if (lastModified > 0) entry.setTime(lastModified);
		if (isDirectory) 
		{
			entry.setMethod(ZipOutputStream.STORED);
			entry.setSize(0);
			entry.setCrc(0);
		}
		else
		{
			entry.setSize(size);
		}
		zip.putNextEntry(entry);
		if (!isDirectory) FileUtil.copy(in, zip);
		zip.flush();
		zip.closeEntry();
	}
	/**
		Finish the ZIP file.
	*/
	@Override
	public void close()
		throws IOException
	{
		zip.close();
		zip = null;
	}
	// -------------------------------------------

}
