/**
	$Id: LabeledExtractImporter.java 5734 2011-09-13 09:41:48Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.batchimport;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.BioMaterialEvent;
import net.sf.basedb.core.BioMaterialEventSource;
import net.sf.basedb.core.BioPlate;
import net.sf.basedb.core.BioWell;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Tag;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;

/**
	Plug-in for importing labled extract items in a batch. The plug-in can create new 
	items and updated existing items.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2011-09-13 11:41:48 +0200 (ti, 13 sep 2011) $
	@deprecated In 3.0, use {@link ExtractImporter} instead
*/
@Deprecated
public class LabeledExtractImporter
	extends AbstractItemImporter<Extract>
{

	private static final Set<GuiContext> guiContexts =  
		Collections.singleton(new GuiContext(Item.EXTRACT, GuiContext.Type.LIST));


	protected static final PluginParameter<String> labelColumnMapping = new PluginParameter<String>(
			"labelColumnMapping",
			"Label",
			"Mapping that picks the name or ID of a label from the data columns. The plug-in will first " +
			"try to find a label with the given name. If none is found and the value is numeric it will " +
			"try to load by internal ID. " +
			"Example: \\Label\\",
			optionalColumnMapping
			);
	
	public LabeledExtractImporter()
	{}

	/*
		From the InteractivePlugin interface
		------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	// ------------------------------------

	/*
		From the AbstractItemImporter class
		------------------------------------
	*/
	
	private Mapper nameMapper;
	private Mapper descriptionMapper;
	private Mapper externalIdMapper;
	private Mapper labelMapper;
	private Mapper originalQuantityMapper;
	private Mapper createdMapper;
	private Mapper protocolMapper;
	private Mapper parentMapper;
	private Mapper usedQuantityMapper;
	private Mapper bioPlateMapper;
	private Mapper bioWellRowMapper;
	private Mapper bioWellColMapper; 
	
	/**
		Useable methods are:
		<ul>
		<li>{@link PropertyIdMethod#NAME}
		<li>{@link PropertyIdMethod#EXTERNAL_ID}
		<li>{@link InternalIdMethod#INTERNAL_ID}
		<li>{@link FallbackIdMethod#NAME_OR_ID}
		</ul>
	*/
	@Override
	protected IdMethod[] getIdMethods()
	{
		return new IdMethod[] { PropertyIdMethod.NAME, PropertyIdMethod.EXTERNAL_ID, 
				InternalIdMethod.INTERNAL_ID, FallbackIdMethod.NAME_OR_ID};
	}
	
	/**
		Adds column mappings for name, externalId, description,
		protocol, creation date, original quantity, pooled, parent
		and used quantity from parent.
	*/
	@Override
	protected void addMoreColumnMappingParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(internalIdColumnMapping);
		parameters.add(nameColumnMapping);
		parameters.add(externalIdColumnMapping);
		parameters.add(labelColumnMapping);
		parameters.add(descriptionColumnMapping);
		parameters.add(protocolColumnMapping);
		
		parameters.add(SampleImporter.createdColumnMapping);
		parameters.add(SampleImporter.originalQuantityColumnMapping);
		parameters.add(SampleImporter.parentColumnMapping);
		parameters.add(SampleImporter.usedQuantityColumnMapping);
		parameters.add(SampleImporter.bioPlateColumnMapping);
		parameters.add(SampleImporter.bioWellRowColumnMapping);
		parameters.add(SampleImporter.bioWellColColumnMapping);
		parameters.add(permissionTemplateColumnMapping);
	}
	
	/**
		Calls {@link Extract#getQuery()} and set a subtype filter for
		{@link Extract#LABELED}.
	*/
	@Override
	protected ItemQuery<Extract> createItemQuery()
	{
		ItemQuery<Extract> query = Extract.getQuery();
		query.restrictPermanent(
				Restrictions.eq(
					Hql.property("itemSubtype"), 
					Expressions.integer(SystemItems.getId(Extract.LABELED))
				)
			);
		return query;
	}
	
	@Override
	protected void createColumnMappers(FlatFileParser ffp, boolean cropStrings)
	{
		nameMapper = getMapper(ffp, (String)job.getValue("nameColumnMapping"), 
				cropStrings ? Extract.MAX_NAME_LENGTH : null, null);
		descriptionMapper = getMapper(ffp, (String)job.getValue("descriptionColumnMapping"), 
				cropStrings ? Extract.MAX_DESCRIPTION_LENGTH : null, null);
		externalIdMapper = getMapper(ffp, (String)job.getValue("externalIdColumnMapping"), 
				cropStrings ? Extract.MAX_EXTERNAL_ID_LENGTH : null, null);
		labelMapper = getMapper(ffp, (String)job.getValue("labelColumnMapping"), null, null);
		originalQuantityMapper = getMapper(ffp, (String)job.getValue("originalQuantityColumnMapping"), null, null);
		createdMapper = getMapper(ffp, (String)job.getValue("createdColumnMapping"), null, null);
		protocolMapper = getMapper(ffp, (String)job.getValue("protocolColumnMapping"), null, null);
		parentMapper = getMapper(ffp, (String)job.getValue("parentColumnMapping"), null, null);
		usedQuantityMapper = getMapper(ffp, (String)job.getValue("usedQuantityColumnMapping"), null, null);
		bioPlateMapper = getMapper(ffp, (String)job.getValue("bioPlateColumnMapping"), null, null);
		bioWellRowMapper = getMapper(ffp, (String)job.getValue("bioWellRowColumnMapping"), null, null);		
		bioWellColMapper = getMapper(ffp, (String)job.getValue("bioWellColColumnMapping"), null, null);
	}
	
	@Override
	protected Extract createItem(DbControl dc, FlatFileParser.Data data)
	{
		Extract e = Extract.getNew(dc);
		e.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(Extract.LABELED)));
		updateItem(dc, e, data);
		return e;
	}
	
	@Override
	protected void updateItem(DbControl dc, Extract extract, FlatFileParser.Data data)
	{
		if (nameMapper != null) extract.setName(nameMapper.getValue(data));
		if (descriptionMapper != null) extract.setDescription(descriptionMapper.getValue(data));
		if (externalIdMapper != null) extract.setExternalId(externalIdMapper.getValue(data));
		if (originalQuantityMapper != null) extract.setOriginalQuantity(originalQuantityMapper.getFloat(data));
		BioMaterialEvent creationEvent = extract.getCreationEvent();
		if (createdMapper != null) creationEvent.setEventDate(parseDate(createdMapper.getValue(data)));

		if (protocolMapper != null) 
		{
			String nameOrId = protocolMapper.getValue(data);
			ItemSubtype type = ItemSubtype.getById(dc, SystemItems.getId(Protocol.LABELING));
			Protocol protocol = findProtocol(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId, type);
			if (nameOrId == null || protocol != null) creationEvent.setProtocol(protocol);
		}
		
		if (labelMapper != null) 
		{
			String nameOrId = labelMapper.getValue(data);
			ItemSubtype labelType = ItemSubtype.getById(dc, SystemItems.getId(Tag.LABEL));
			Tag label = findTag(dc, FallbackIdMethod.NAME_OR_ID, labelMapper.getValue(data), labelType);
			if (nameOrId == null || label != null) extract.setTag(label);
		}
		
		if (bioPlateMapper != null && bioWellRowMapper != null && bioWellColMapper != null)
		{
			String plateNameOrId = bioPlateMapper.getValue(data);
			String row = bioWellRowMapper.getValue(data);
			String column = bioWellColMapper.getValue(data);
			BioPlate plate = findBioPlate(dc, FallbackIdMethod.NAME_OR_ID, plateNameOrId);
			BioWell well = findBioWell(dc, plate, row, column);
			if (plateNameOrId == null || row == null || column == null || well != null)
			{
				extract.setBioWell(well);
			}
		}

		if (parentMapper != null)
		{
			String nameOrId = parentMapper.getValue(data);
			Extract parent = findExtract(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId);
			Float usedQuantity = usedQuantityMapper == null ?
				null : usedQuantityMapper.getFloat(data);
			if (nameOrId == null || parent != null) 
			{
				BioMaterialEventSource evtSrc = extract.getCreationEvent().setSource(parent);
				if (evtSrc != null) evtSrc.setUsedQuantity(usedQuantity);
			}
		}
	}
	
	/**
		If the labeled extract is pooled, add the parent labeled extract.
	*/
	@Override
	protected void updateMultiLineItem(DbControl dc, Extract extract, 
		FlatFileParser.Data data, int multiLineNum)
	{
		if (parentMapper != null)
		{
			String nameOrId = parentMapper.getValue(data);
			Extract parent = findLabeledExtract(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId);
			if (parent != null)
			{
				Float usedQuantity = usedQuantityMapper == null ?
						null : usedQuantityMapper.getFloat(data);
				extract.getCreationEvent().addSource(parent).setUsedQuantity(usedQuantity);
				// Use same tag as parent if not yet set and not mapped in the file
				if (labelMapper == null && parent.getTag() != null && extract.getTag() == null)
				{
					extract.setTag(parent.getTag());
				}
			}
		}
	}
	// ------------------------------------
}
