/*
	$Id$

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.List;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.plugins.IntensityCalculatorPlugin;

public class TestIntensityCalculatorPlugin
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		write("++Testing intensity calculator plug-in");
		try
		{
			int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
			int formulaId = TestAnalyzeUtil.getItemId("formula.intensity.genepix.mean");
			if (experimentId == 0 || formulaId == 0)
			{
				ok = false;
				write("Test experiment was not setup correctly.");
				return ok;
			}
	
			int jobId = test_create_job(experimentId, formulaId, "Intensity calculator plugin");
			ok &= TestJob.test_execute(jobId, false);
			int rootBasId = TestAnalyzeUtil.getBioAssaySetIdByName(experimentId, "Intensity calculator plugin");
			TestAnalyzeUtil.setItemId("bioassayset.root.intensity-calculator-plugin", rootBasId);
		}
		finally
		{
			if (wasSetup) TestAnalyzeUtil.cleanupExperiment();
			write("++Testing intensity calculator plug-in "+(ok ? "OK" : "Failed")+"\n");
		}
		return ok;
	}
	
	static int test_create_job(int experimentId, int formulaId, String rootName)
	{
		if (experimentId == 0 || formulaId == 0) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, IntensityCalculatorPlugin.class.getName());
			Experiment experiment = Experiment.getById(dc, experimentId);
			Formula formula = Formula.getById(dc, formulaId);
			List<RawBioAssay> rawBioAssays = experiment.getRawBioAssays().list(dc);
			dc.getSessionControl().getCurrentContext(Item.EXPERIMENT).setId(experimentId);
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("Calculating root intensities using ");
			
			PluginConfigurationRequest request = job.configure(null);
			request.setParameterValue("experiment", experiment);
			request.setParameterValue("name", rootName);
			request.setParameterValue("formula", formula);
			request.setParameterValues("rawBioAssays", rawBioAssays);

			TestJob.write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(job);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = job.getId();
			
			
			write("--Create job for root intensity calculation OK");

		}
		catch (Throwable ex)
		{
			write("--Create job for root intensity calculation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}


	static void write(String message)
	{
		System.out.println(message);
	}
}
