/*
	$Id: TestSpotImages.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;

import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.media.jai.JAI;
import com.sun.media.jai.widget.DisplayJAI;
import com.sun.media.jai.codec.SeekableStream;
import javax.media.jai.RenderedOp;
import java.awt.image.renderable.ParameterBlock;
import javax.swing.JFrame;
import javax.swing.Box;

public class TestSpotImages
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing spot images import");
		
		int rawBioAssayId = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, 0, 0, false);
		TestReporter.test_import_from_file("data/test.rawdata.import.txt", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);
		TestRawBioAssay.test_import_from_file(rawBioAssayId, "data/test.rawdata.import.txt");
		
		int imageFileId = TestFile.test_create("data/spots.tif", false, true);
		int spotImagesFileId = TestFile.test_create("test"+System.currentTimeMillis()+".zip", false, true);
		test_create_spotimages(rawBioAssayId, imageFileId, spotImagesFileId);

		test_fetch_spot(rawBioAssayId, 1);
		ArrayList<Integer> positions = new ArrayList<Integer>(5);
		positions.add(1);
		positions.add(110);
		positions.add(253);
		positions.add(260);
		positions.add(519);
		test_multi_fetch_spot(rawBioAssayId, positions);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestRawBioAssay.test_delete(rawBioAssayId);
		TestFile.test_delete(imageFileId);
		TestFile.test_delete(spotImagesFileId);
		TestReporter.test_delete();

		write("++Testing spot images import "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_create_spotimages(int rawbioassayId, int imageFileId, int spotImagesFileId)
	{
		if (rawbioassayId == 0 || imageFileId == 0) return;
		long time = 0;
		write("--Start create spot images");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			RawBioAssay rba = RawBioAssay.getById(dc, rawbioassayId);
			File image = File.getById(dc, imageFileId);
			File spots = File.getById(dc, spotImagesFileId);
			
			time = System.currentTimeMillis();
			SpotImages si = rba.getSpotImages();
			ProgressReporter progress = TestUtil.getSilent() ? null : new ProgressReporterImpl();
			si.createSpotImages(spots, 5, 5, 0, 0, 32, 1.8f, 0.75f, image, null, image, progress);
			dc.commit();
			time = System.currentTimeMillis()-time;
			write("--Create spot images OK (time="+time+" ms)");
		}
		catch (Exception ex)
		{
			write("--Create spot images FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_fetch_spot(int rawBioAssayId, int position)
	{
		if (rawBioAssayId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			RawBioAssay rba = RawBioAssay.getById(dc, rawBioAssayId);
			
			DataQuery<RawData> query = rba.getRawData();
			query.restrict(
				Restrictions.eq(
					Hql.property("position"),
					Expressions.integer(position)
				)
			);
			
			
			RawData rd = query.iterate(dc).next();
			SpotImages si = rba.getSpotImages();
			SpotImage spot = si.getSpotImage(rd);
				
			SeekableStream stream = SeekableStream.wrapInputStream(spot.getInputStream(), true);
			ParameterBlock pngParameters = new ParameterBlock();
			pngParameters.add(stream);
			RenderedOp image = JAI.create("png", pngParameters);
			
			if (!TestUtil.getSilent())
			{
				JFrame window = new JFrame("Show spot");
				window.getContentPane().add(new DisplayJAI(image));
				window.pack();
				window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				window.setVisible(true);
			}
			write("--Fetching spot image OK");
		}
		catch (Exception ex)
		{
			write("--Fetching spot image FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_multi_fetch_spot(int rawBioAssayId, List<Integer> positions)
	{
		if (rawBioAssayId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			RawBioAssay rba = RawBioAssay.getById(dc, rawBioAssayId);
			
			DataQuery<RawData> query = rba.getRawData();
			query.restrict(
				Restrictions.in(
					Hql.property("position"),
					Expressions.parameter("positions")
				)
			);
			query.setParameter("positions", positions, Type.INT);
			DataResultIterator<RawData> rawData = query.iterate(dc);
			List<RawData> rawDataList = new LinkedList<RawData>();
			while (rawData.hasNext())
			{
				rawDataList.add(rawData.next());
			}
			
			List<SpotImage> spotImages = rba.getSpotImages().getSpotImages(rawDataList);

			JFrame window = null;
			if (!TestUtil.getSilent())
			{
				window = new JFrame("Show spots");
				window.setBounds(0, 100, 250, 50);
				Box box = Box.createHorizontalBox();
				window.setContentPane(box);
				window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				window.setVisible(true);
			}
			for (SpotImage spot : spotImages)
			{
				SeekableStream stream = SeekableStream.wrapInputStream(spot.getInputStream(), true);
				ParameterBlock pngParameters = new ParameterBlock();
				pngParameters.add(stream);
				RenderedOp image = JAI.create("png", pngParameters);

				if (!TestUtil.getSilent())
				{
					DisplayJAI display = new DisplayJAI(image);
					window.getContentPane().add(display);
					window.validate();
				}
			}

			write("--Fetching multiple spot images OK ("+rawDataList.size()+")");
		}
		catch (Exception ex)
		{
			write("--Fetching multiple spot images FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static class ProgressReporterImpl
		implements ProgressReporter
	{
		@Override
		public void display(int percent, String message)
		{
			System.out.println("--SpotImages: "+message);
		}

		@Override
		public void append(String message)
		{}
	}
}

