/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Wells = function()
{
	var wells = {};
	
	/**
		Initialize the page.
	*/
	wells.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', wells.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', Annotations.onSwitchToAnnotationsTab);
			TabControl.addTabValidator('settings.info', wells.validateWell);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'WELL', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'WELL'};
			var tableAttributes = {'table-id': 'wells'};
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewReporterList', wells.newReporterList);

			TabControl.setSwitchFunction('main', wells.switchTab);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	wells.newReporterList = function()
	{
		var controller = Items.getController('REPORTERLIST');
		Table.submitToPopup('wells', 'CreateReporterList', controller.width, controller.height);
	}
	
	wells.switchTab = function(event)
	{
		var target = event.target;
		var tabId = target.id.substring(5);
		var frm = document.forms['wells'];
		var plateId = frm.plate_id.value;

		var url;
		if (tabId == 'events')
		{
			url = '../events/index.jsp?ID='+App.getSessionId();
			url += '&plate_id='+plateId;
		}
		else
		{
			url = '../index.jsp?ID='+App.getSessionId();
			url += '&cmd=ViewItem&item_id='+plateId;
			url += '&tab='+tabId;
		}
		location.replace(url);
	}

	wells.validateWell = function()
	{
		return true;
	}

	// Submit the form
	wells.save = function()
	{
		var frm = document.forms['well'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			frm.submit();
		}
	}
	
	return wells;
}();

Doc.onLoad(Wells.initPage);
