/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Login = function()
{
	var pLoginForm = 'login';
	var pUseLastLogin = 1;
	
	var login = {};
	
	login.initPage = function()
	{
		// Information links
		Buttons.addClickHandler('aboutServer', login.showAbout);
		Buttons.addClickHandler('getAccount', login.showGetAccount);
		Buttons.addClickHandler('forgotPassword', login.showForgotPassword);
		
		// Login buttons
		Events.doOnEnter('login', login.setFocus);
		Buttons.addClickHandler('btnLogin', login.doLogin);
		Buttons.addClickHandler('btnLoginAnyway', login.enableLogin);
		Buttons.addClickHandler('close', App.closeWindow);
		
		// Password field should login if 'ENTER' is pressed
		Events.doOnEnter('password', login.doLogin);
		
		login.initForm();
		
		var footNoteFrame = App.topWindow().frames['footnote'];
		if (footNoteFrame)
		{
			footNoteFrame.location.reload();			
		}
		
		if (Exception) Exception.fixWindow();
	}
	
	/**
		Initialize the login form by populating it with the last used login
		and setting focus to either the login or password field.
	*/
	login.initForm = function()
	{
		var frm = document.forms[pLoginForm];
		pUseLastLogin = Data.int(frm.login, 'use-last-login', 1);
		if (pUseLastLogin)
		{
			var lastLogin = this.getLastLogin();
			if (!frm.login.value && lastLogin) 
			{
				frm.login.value = lastLogin;
				Doc.show('timeout');
			}
		}
		login.setFocus();
	}

	
	/*
		Save the last login. If local storage is available we use that,
		otherwise we simply save it on the top frameset.
	*/
	login.saveLastLogin = function(lastLogin)
	{
		var storage = App.localStorage();
		if (storage)
		{
			App.setLocal('last-login', lastLogin);
		}
		else
		{
			App.topWindow().lastLogin = lastLogin;
		}
	}
	
	/**
		Get the last used login. If local storage is available we
		use that, otherwise we use the top frameset.
	 */
	login.getLastLogin = function()
	{
		var lastLogin;
		var storage = App.localStorage();
		if (storage)
		{
			lastLogin = App.getLocal('last-login');
		}
		if (!lastLogin)
		{
			lastLogin = App.topWindow().lastLogin;
		}
		return lastLogin;
	}
	
	/**
		Set focus on the login field if empty, otherwise the password field.
	*/
	login.setFocus = function()
	{
		var frm = document.forms[pLoginForm];
		if (!frm.login.value)
		{
			frm.login.focus();
			frm.login.select();
		}
		else
		{
			frm.password.focus();
			frm.password.select();
		}
	}
	
	/**
		Enable the (disabled) login form.
	*/
	login.enableLogin = function()
	{
		var frm = document.forms[pLoginForm];
		frm.login.disabled = false;
		frm.password.disabled = false;
		Doc.hide('btnLoginAnyway');
		Doc.removeClass('btnLogin', 'disabled');
		this.setFocus();
	}
	
	/**
		Send the login form.
	*/
	login.doLogin = function(event)
	{
		var frm = document.forms[pLoginForm];
		// Check if login has been disabled
		if (frm.login && frm.login.disabled) return;

		// Check if the login is not recommended
		var denyLogin = Doc.element('denyLogin');
		if (denyLogin)
		{
			if (!confirm('Login has been disabled by an administrator. Do you want to continue anyway?'))
			{
				return;
			}
		}
		
		// On the impersonate form, check that a user has been selected
		if (frm.user_id && !frm.user_id.value)
		{
			Forms.showNotification('user_id.select', 'Please select a user');
			return;
		}

		// Store the last login
		if (pUseLastLogin && frm.login)
		{
			Login.saveLastLogin(frm.login.value);
		}
		
		// Check 'remain on page' and 'redirect' options
		if (frm.remainOnPage && frm.redirect)
		{
			if (frm.remainOnPage.checked) 
			{
				frm.redirect.value = window.opener ? window.opener.location.href : '';
			}
			else
			{
				frm.redirect.value = App.getRoot()+'my_base/index.jsp?ID='+getSessionId();
			}
		}
		
		if (frm.target) Dialogs.openPopup('', frm.target, 300, 200);
		if (!pUseLastLogin)
		{
			// Duplicate in hidden form
			var frm2 = Forms.cloneAsHidden(frm);
			document.body.appendChild(frm2);
			frm2.submit();
			document.body.removeChild(frm2);
		}
		else
		{
			frm.submit();
		}
		
	}
	
	/**
		Show popup dialog with information about this server.
	*/
	login.showAbout = function()
	{
		var url = App.getRoot()+'info/about.jsp?ID='+App.getSessionId();
		url += '&page=about';
		Dialogs.openPopup(url, 'About', 600, 400);
	}
	
	/**
		Show popup dialog with information about how to get an account on this server.
	*/
	login.showGetAccount = function()
	{
		var url = App.getRoot()+'info/get_account.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'GetAccount', 450, 300);
	}
	
	/**
		Show popup dialog with information about what to do if a password has been lost.
	*/
	login.showForgotPassword = function()
	{
		var url = App.getRoot()+'info/forgot_password.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'ForgotPassword', 450, 300);
	}
	
	return login;
}();

Doc.onLoad(Login.initPage);

var Impersonate = function()
{
	var pForm = 'login';
	var impersonate = {};
	
	impersonate.initPage = function()
	{
		Buttons.addClickHandler('user_id.select', impersonate.selectUser);
		Events.addEventHandler('user_id', 'base-selected', impersonate.setUserCallback);
	}
	
	/**
		Open the 'select user' dialog.
	*/
	impersonate.selectUser = function()
	{
		var frm = document.forms[pForm];
		var url = '&resetTemporary=1';
		if (frm.user_id.length > 0)
		{
			var id = Math.abs(parseInt(frm.user_id[0].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('USER', 'user_id', 0, url);
	}
	
	/**
		Callback for setting the selected user.
	*/
	impersonate.setUserCallback = function(event)
	{
		var frm = document.forms[pForm];
		var list = frm.user_id;
		if (list.length < 1 || list[0].value == '0')
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
	}

	return impersonate;
}();

Doc.onLoad(Impersonate.initPage);
