/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Reporters = function()
{
	var reporters = {};
	var extendedProperties;
	
	/**
		Initialize the page.
	*/
	reporters.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', reporters.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', reporters.validateReporter);
			TabControl.addTabValidator('settings.extended', reporters.validateExtendedProperties);
			
			// Extended properties
			extendedProperties = Data.json('page-data', 'extended-properties');
			for (var i = 0; i < extendedProperties.length; i++)
			{
				var ep = extendedProperties[i];
				var valueType = ep.valueType;
				var fieldName = 'ep.'+ep.name;
				
				if (valueType == 'INT' || valueType == 'LONG')
				{
					// Block non-numeric keys from INT and LONG fields
					Events.addEventHandler(fieldName, 'keypress', Events.integerOnly);
				}
				else if (valueType == 'FLOAT' || valueType == 'DOUBLE')
				{
					// Block non-numeric keys from FLOAT and DOUBLE fields
					Events.addEventHandler(fieldName, 'keypress', Events.numberOnly);
				}
			}

			
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'REPORTER', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'REPORTER'};
			var tableAttributes = {'table-id': 'reporters'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewReporterList', reporters.newReporterList);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	reporters.newReporterList = function()
	{
		var controller = Items.getController('REPORTERLIST');
		Table.submitToPopup('reporters', 'CreateReporterList', controller.width, controller.height);
	}
	
	reporters.validateReporter = function()
	{
		var frm = document.forms['reporter'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name for the reporter');
			return false;
		}
		else if (Strings.trim(frm.externalId.value) == '')
		{
			Forms.showNotification(frm.externalId, 'You must enter an external ID for the reporter');
			return false;
		}
		return true;
	}

	
	// Validate the 'Extended properties' tab with extended properties
	reporters.validateExtendedProperties = function()
	{
		var frm = document.forms['user'];
		var dateFormat = Data.get('page-data', 'date-format');
		var dateTimeFormat = Data.get('page-data', 'datetime-format');
		
		for (var i = 0; i < extendedProperties.length; i++)
		{
			var ep = extendedProperties[i];
			var valueType = ep.valueType;
			var field = frm['ep.'+ep.name];
			
			if (ep.valueType == 'DATE')
			{
				if (field.value != '' && !Dates.isDate(field.value, dateFormat))
				{
					Forms.showNotification(field, "'"+field.value+"' is not a valid date for '" + ep.title + "'");
					return false;
				}
			}
			else if (ep.valueType == 'TIMESTAMP')
			{
				if (field.value != '' && !Dates.isDate(field.value, dateTimeFormat))
				{
					Forms.showNotification(field, "'"+field.value+"' is not a valid timestamp for '" + ep.title + "'");
					return false;
				}
			}
			if (!ep.nullable)
			{
				if (ep.valueType == 'BOOLEAN')
				{
					if (Forms.getCheckedRadio(field) == null)
					{
						Forms.showNotification(field[0], "You must select a value for '" + ep.title + "'");
						return false;
					}
				}
				else
				{
					if (Strings.trim(field.value) == '')
					{
						Forms.showNotification(field, "You must enter a value for '" + ep.title + "'");
						return false;
					}
				}
			}
		}	
		return true;
	}

	reporters.save = function()
	{
		var frm = document.forms['reporter'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}


	return reporters;
}();

Doc.onLoad(Reporters.initPage);
